/****************************************************************************

    MSort : Merges two sorted wordlists keeping them sorted.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Programmi gestione wordlists.
  File        : MSort.c
  Autore      : Sbragion Denis
  Descrizione : Effettua la combinazione di due liste ordinate mantenendo
                l' ordine.
  Revisioni   :
  11/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License. Modifiche per
                l' inserimento di un fine linea sull' ultima riga.
                Eliminazione dei problemi di bufferizzazione.
  27/6/94     : Correzione di alcuni messaggi di errore.
                Riduzione dell' occupazione dello stack.
****************************************************************************/

#include "boolean.h"
#include "strcomp.h"
#include <string.h>

#define MaxLineLength (MaxStringLength + 1)
#define WriteModulus 1000

typedef enum { None, String1, String2 } StringIdType;

int main(int argc,char * argv[])
  {
    static char Str1[MaxLineLength];
    static char Str2[MaxLineLength];
    FILE * InFile1;
    FILE * InFile2;
    FILE * OutFile;
    Boolean InFile1Eof = False;
    Boolean InFile2Eof = False;
    StringIdType StringToWrite = None;
    unsigned long int LineCount = 0;

    if (argc < 4)
      {
        puts("MSort : Merges two sorted wordlists keeping them sorted.");
        puts("Usage : MSort InputFile1 InputFile2 OutputFile");
        return((int) Ok);
      }

    if ((InFile1 = fopen(argv[1],"rt")) == NULL)
      {
        puts("Error : Can't open input file 1.");
        puts("Usage : MSort InputFile1 InputFile2 OutputFile");
        return((int) Error);
      }

    if ((InFile2 = fopen(argv[2],"rt")) == NULL)
      {
        puts("Error : Can't open input file 2.");
        puts("Usage : MSort InputFile1 InputFile2 OutputFile");
        fclose(InFile1);
        return((int) Error);
      }

    if ((OutFile = fopen(argv[3],"wt")) == NULL)
      {
        puts("Error : Can't open output file.");
        puts("Usage : MSort InputFile1 InputFile2 OutputFile");
        fclose(InFile1);
        fclose(InFile2);
        return((int) Error);
      }

    printf("Merging : ");
    fflush(stdout);

    do
      {
        if (StringToWrite == None)
          {
            if (fgets(Str1,MaxLineLength,InFile1) == NULL)
              InFile1Eof = True;
            else
              StripEndOfLine(Str1);

            if (fgets(Str2,MaxLineLength,InFile2) == NULL)
              InFile2Eof = True;
            else
              StripEndOfLine(Str2);
          }
        else
          {
            if (StringToWrite == String1)
              if (fgets(Str1,MaxLineLength,InFile1) == NULL)
                InFile1Eof = True;
              else
                StripEndOfLine(Str1);

            if (StringToWrite == String2)
              if (fgets(Str2,MaxLineLength,InFile2) == NULL)
                InFile2Eof = True;
              else
                StripEndOfLine(Str2);
          }

        if (InFile1Eof && !InFile2Eof)
          StringToWrite = String2;

        if (InFile2Eof && !InFile1Eof)
          StringToWrite = String1;

        if (InFile1Eof && InFile2Eof)
          StringToWrite = None;

        if (!InFile1Eof && !InFile2Eof)
          if (strcmp(Str1,Str2) <= 0)
            StringToWrite = String1;
          else
            StringToWrite = String2;

        switch (StringToWrite)
          {
            case String1 :
              if (fprintf(OutFile,"%s\n",Str1) == EOF)
                {
                  puts("\nError : Can't write to output file.");
                  fclose(InFile1);
                  fclose(InFile2);
                  fclose(OutFile);
                  return((int) Error);
                }

              LineCount++;
              break;

            case String2 :
              if (fprintf(OutFile,"%s\n",Str2) == EOF)
                {
                  puts("\nError : Can't write to output file.");
                  fclose(InFile1);
                  fclose(InFile2);
                  fclose(OutFile);
                  return((int) Error);
                }

              LineCount++;
              break;

            case None :
              /* Fine del file. */
              break;
          }

        if ((LineCount % WriteModulus) == 0)
          {
            printf("*");
            fflush(stdout);
          }
      }
    while (StringToWrite != None);

    fclose(InFile1);
    fclose(InFile2);
    fclose(OutFile);

    printf("\n%lu Lines.\n",LineCount);
    puts("Done.");
    return((int) Ok);
  }

/***************************************************************************/
