/****************************************************************************

    Split : Split a wordlist into file of N lines.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Programmi gestione wordlists.
  File        : Split.c
  Autore      : Sbragion Denis
  Descrizione : Spezza una wordlist in file di n linee.
  Revisioni   :
  12/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License. Modifiche per
                l' inserimento di un fine linea sull' ultima riga.
                Eliminazione dei problemi di bufferizzazione.
  27/6/94     : Correzione di alcuni messaggi di errore.
                Riduzione dell' occupazione dello stack.
****************************************************************************/

#include "boolean.h"
#include "strcomp.h"
#include <stdlib.h>
#include <string.h>

#define MaxLineLength (MaxStringLength + 1)
#define WriteModulus 1000

#define ExtensionSize 4
#define BaseNameSize 10
#define FileCountLimit 999

int main(int argc,char * argv[])
  {
    static char ReadStr[MaxLineLength];
    static char Extension[ExtensionSize];
    static char BaseName[BaseNameSize] = "split.";
    static char FileName[BaseNameSize + ExtensionSize - 1];
    FILE * InFile;
    FILE * OutFile;
    unsigned long int LineCount = 0;
    unsigned long int SplitSize;
    unsigned long int FileLineCount = 0;
    unsigned int FileCount = 0;
    Boolean FirstLine = True;

    if (argc < 3)
      {
        puts("Split : Split a wordlist into file of N lines.");
        puts("Usage : Split N InputFile [BaseName]");
        return((int) Ok);
      }

    if ((SplitSize = (unsigned long int) atol(argv[1])) == 0)
      {
        puts("Error : Can't convert split size.");
        puts("Usage : Split N InputFile [BaseName]");
        return((int) Error);
      }

    if ((InFile = fopen(argv[2],"rt")) == NULL)
      {
        puts("Error : Can't open input file.");
        puts("Usage : Split N InputFile [BaseName]");
        return((int) Error);
      }

    if (argc > 3)
      {
        strncpy(BaseName,argv[3],(size_t) BaseNameSize - 1);
        BaseName[BaseNameSize - 1] = '\0';
      }

    printf("Splitting : ");
    fflush(stdout);

    while (fgets(ReadStr,MaxLineLength,InFile) != NULL)
      {
        if (FirstLine)
          {
            FirstLine = False;
            strcpy(FileName,BaseName);
            sprintf(Extension,"%03u",FileCount);
            strcat(FileName,Extension);

            if ((OutFile = fopen(FileName,"wt")) == NULL)
              {
                puts("\nError : Can't open output file.");
                puts("Usage : Split N InputFile [BaseName]");
                fclose(InFile);
                return((int) Error);
              }
          }

        StripEndOfLine(ReadStr);

        if (fprintf(OutFile,"%s\n",ReadStr) == EOF)
          {
            puts("\nError : Can't write to output file.");
            fclose(InFile);
            fclose(OutFile);
            return((int) Error);
          }

        FileLineCount++;
        if (FileLineCount >= SplitSize)
          {
            FirstLine = True;
            FileLineCount = 0;
            fclose(OutFile);

            FileCount++;
            if (FileCount > FileCountLimit)
              {
                puts("\nError : Too many split files.");
                fclose(InFile);
                return((int) Error);
              }
          }

        LineCount++;
        if ((LineCount % WriteModulus) == 0)
          {
            printf("*");
            fflush(stdout);
          }
      }

    fclose(InFile);
    if (!FirstLine)
      fclose(OutFile);

    printf("\n%lu Lines.\n",LineCount);
    puts("Done.");
    return((int) Ok);
  }

/***************************************************************************/
