/****************************************************************************

    StrComp : Wordlist compression library.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Libreria compressione wordlists.
  File        : StrComp.c
  Autore      : Sbragion Denis
  Descrizione : Funzioni di base per la compressione di stringhe.
  Revisioni   :
  11/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License.
****************************************************************************/

#include "strcomp.h"
#include <string.h>

/* Comprime la stringa Str supponendo che la stringa PStr sia la stringa
che precede Str nella wordlist. Pone il risultato in CStr. */
void StringCompress(char * Str,char * PStr,Byte * CStr)
  {
    unsigned int StrLen = strlen(Str);
    unsigned int PStrLen = strlen(PStr);
    unsigned int L;
    unsigned int I;
    unsigned int J;

    if (StrLen > PStrLen)
      L = PStrLen;
    else
      L = StrLen;

    for (I = 0;(I < L) && (Str[I] == PStr[I]);I++);

    CStr[0] = (Byte) I;
    CStr[1] = (Byte) StrLen - I;

    for(J = 2;I < StrLen;I++,J++)
      CStr[J] = (Byte) Str[I];
  }

/* Espande la stringa compressa CStr supponendo che la stringa PStr sia la
stringa che precede Str nella wordlist. Pone il risultato in Str. */
void StringExpand(Byte * CStr,char * PStr,char * Str)
  {
    unsigned int I;
    unsigned int J;
    unsigned int EL = (unsigned int) CStr[0];
    unsigned int DL = (unsigned int) CStr[1] + 2;

    for (I = 0;I < EL;I++)
      Str[I] = PStr[I];

    for(J = 2;J < DL;J++,I++)
      Str[I] = (char) CStr[J];

    Str[I] = '\0';
  }

/* Espande la stringa compressa CStr e pone il risultato in Str che,
inizialmente deve contenere la stringa precedente nella wordlist. */
void StringUpdate(Byte * CStr,char * Str)
  {
    unsigned int J;
    unsigned int I = (unsigned int) CStr[0];
    unsigned int DL = (unsigned int) CStr[1] + 2;

    for(J = 2;J < DL;J++,I++)
      Str[I] = (char) CStr[J];

    Str[I] = '\0';
  }

/* Legge la stringa compressa CStr dal file F che si suppone gia`
aperto in modo binario ("rb"). */
ErrorType ReadCompressedString(Byte * CStr,FILE * F)
  {
    size_t DL;

    if (fread((void *) CStr,sizeof(Byte),(size_t) 2,F) != (size_t) 2)
      return(Error);

    DL = (size_t) CStr[1];
    if (fread((void *) &CStr[2],sizeof(Byte),DL,F) != DL)
      return(Error);

    return(Ok);
  }

/* Scrive la stringa compressa CStr nel file F che si suppone gia`
aperto in modo binario ("wb"). */
ErrorType WriteCompressedString(Byte * CStr,FILE * F)
  {
    size_t S = (size_t) CStr[1] + 2;

    if (fwrite((void *) CStr,sizeof(Byte),S,F) != S)
      return(Error);

    return(Ok);
  }

/* Elimina lo '\n' finale dalla stringa S se presente. */
void StripEndOfLine(char * S)
  {
    char * EOLPos = strrchr(S,(int) '\n');

    if (EOLPos != NULL)
      *EOLPos = '\0';
  }

/***************************************************************************/
