/****************************************************************************

    StrComp : Wordlist compression library.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Libreria compressione wordlists.
  File        : StrComp.h
  Autore      : Sbragion Denis
  Descrizione : Funzioni di base pe la compressione di stringhe.
  Revisioni   :
  11/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License.
****************************************************************************/

#ifndef StrComp_h
  #define StrComp_h

  #include "error.h"
  #include <stdio.h>

  /* Lunghezza massima per una stringa da comprimere o espandere. */
  #define MaxStringLength 255

  /* Tipo corrispondente ad un byte. Le norme ANSI non assicurano
  che un unsigned char corrisponda ad un byte. Utilizzare eventualmente
  un altro tipo, purche` corrisponda ad un byte. */
  typedef unsigned char Byte;

  /* Comprime la stringa Str supponendo che la stringa PStr sia la stringa
  che precede Str nella wordlist. Pone il risultato in CStr. */
  void StringCompress(char * Str,char * PStr,Byte * CStr);

  /* Espande la stringa compressa CStr supponendo che la stringa PStr sia la
  stringa che precede Str nella wordlist. Pone il risultato in Str. */
  void StringExpand(Byte * CStr,char * PStr,char * Str);

  /* Espande la stringa compressa CStr e pone il risultato in Str che,
  inizialmente deve contenere la stringa precedente nella wordlist. */
  void StringUpdate(Byte * CStr,char * Str);

  /* Legge la stringa compressa CStr dal file F che si suppone gia`
  aperto in modo binario ("rb"). */
  ErrorType ReadCompressedString(Byte * CStr,FILE * F);

  /* Scrive la stringa compressa CStr nel file F che si suppone gia`
  aperto in modo binario ("wb"). */
  ErrorType WriteCompressedString(Byte * CStr,FILE * F);

  /* Elimina lo '\n' finale dalla stringa S se presente. */
  void StripEndOfLine(char * S);

#endif

/***************************************************************************/
