/****************************************************************************

    Unique : Strips multiple words from a sorted wordlist.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Programmi gestione wordlists.
  File        : Unique.c
  Autore      : Sbragion Denis
  Descrizione : Toglie le parole multiple da una wordlist ordinata.
  Revisioni   :
  11/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License. Modifiche per
                l' inserimento di un fine linea sull' ultima riga.
                Eliminazione dei problemi di bufferizzazione.
  27/6/94     : Correzione di alcuni messaggi di errore.
                Riduzione dell' occupazione dello stack.
****************************************************************************/

#include "boolean.h"
#include "strcomp.h"
#include <string.h>

#define MaxLineLength (MaxStringLength + 1)
#define WriteModulus 1000

int main(int argc,char * argv[])
  {
    static char Str1[MaxLineLength] = "";
    static char Str2[MaxLineLength] = "";
    char * ReadStr = Str1;
    char * LastStr = Str2;
    char * Tmp;
    FILE * InFile;
    FILE * OutFile;
    unsigned long int LineCount = 0;
    unsigned long int StripCount = 0;

    if (argc < 3)
      {
        puts("Unique : Strips multiple words from a sorted wordlist.");
        puts("Usage  : Unique InputFile OutputFile");
        return((int) Ok);
      }

    if ((InFile = fopen(argv[1],"rt")) == NULL)
      {
        puts("Error : Can't open input file.");
        puts("Usage : Unique InputFile OutputFile");
        return((int) Error);
      }

    if ((OutFile = fopen(argv[2],"wt")) == NULL)
      {
        puts("Error : Can't open output file.");
        puts("Usage : Unique InputFile OutputFile");
        fclose(InFile);
        return((int) Error);
      }

    printf("Stripping : ");
    fflush(stdout);

    while (fgets(ReadStr,MaxLineLength,InFile) != NULL)
      {
        StripEndOfLine(ReadStr);

        if (strcmp(ReadStr,LastStr) == 0)
          StripCount++;
        else
          {
            if (fprintf(OutFile,"%s\n",ReadStr) == EOF)
              {
                puts("\nError : Can't write to output file.");
                fclose(InFile);
                fclose(OutFile);
                return((int) Error);
              }

            Tmp = LastStr;
            LastStr = ReadStr;
            ReadStr = Tmp;
          }

        LineCount++;
        if ((LineCount % WriteModulus) == 0)
          {
            printf("*");
            fflush(stdout);
          }
      }

    fclose(InFile);
    fclose(OutFile);

    printf("\n%lu Lines.\n",LineCount);
    printf("%lu Stripped.\n",StripCount);
    puts("Done.");
    return((int) Ok);
  }

/***************************************************************************/
