/****************************************************************************

    Wle : Wordlists expander.
    Copyright (C) 1994  Sbragion Denis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the author on Internet at the following addresses :

      sbragion@ghost.sm.dsi.unimi.it (preferred)
      denissbr@ipo.sm.dsi.unimi.it

    or through standard mail at :

      Sbragion Denis
      Via E. Fermi 18
      20046 Biassono (MI)
        ITALY

****************************************************************************/

/****************************************************************************
  Progetto    : Programmi gestione wordlists.
  File        : Wle.c
  Autore      : Sbragion Denis
  Descrizione : Effettua l' espansione di una wordlist.
  Revisioni   :
  11/4/94     : Prima stesura.
  16/6/94     : Aggiunta la GNU General Public License. Modifiche per
                l' inserimento di un fine linea sull' ultima riga.
                Eliminazione dei problemi di bufferizzazione.
  27/6/94     : Correzione di alcuni messaggi di errore.
                Riduzione dell' occupazione dello stack.
****************************************************************************/

#include "boolean.h"
#include "strcomp.h"
#include <string.h>

#define MaxLineLength (MaxStringLength + 1)
#define WriteModulus 1000

int main(int argc,char * argv[])
  {
    static char WriteStr[MaxLineLength] = "";
    static Byte CompressedStr[MaxStringLength + 2];
    FILE * InFile;
    FILE * OutFile;
    unsigned long int LineCount = 0;

    if (argc < 3)
      {
        puts("Wle   : Wordlists expander.");
        puts("Usage : Wle InputFile OutputFile");
        return((int) Ok);
      }

    if ((InFile = fopen(argv[1],"rb")) == NULL)
      {
        puts("Error : Can't open input file.");
        puts("Usage : Wle InputFile OutputFile");
        return((int) Error);
      }

    if ((OutFile = fopen(argv[2],"wt")) == NULL)
      {
        puts("Error : Can't open output file.");
        puts("Usage : Wle InputFile OutputFile");
        fclose(InFile);
        return((int) Error);
      }

    printf("Expanding : ");
    fflush(stdout);

    while (ReadCompressedString(CompressedStr,InFile) == Ok)
      {
        StringUpdate(CompressedStr,WriteStr);

        if (fprintf(OutFile,"%s\n",WriteStr) == EOF)
          {
            puts("\nError : Can't write to output file.");
            fclose(InFile);
            fclose(OutFile);
            return((int) Error);
          }

        LineCount++;
        if ((LineCount % WriteModulus) == 0)
          {
            printf("*");
            fflush(stdout);
          }
      }

    fclose(InFile);
    fclose(OutFile);

    printf("\n%lu Lines.\n",LineCount);
    puts("Done.");
    return((int) Ok);
  }

/***************************************************************************/
